from typing import List
from pydantic import BaseModel
import yaml
import os

class Stack(BaseModel):
    path: str
    name: str

class Enviroment(BaseModel):
    name: str
    profile: str
    region: str
    artifacts: str

class Config(BaseModel):
    Enviroments: List[Enviroment]
    Stacks: List[Stack]

    @staticmethod
    def parse(path: str):
        try:
            data = yaml.safe_load(open(path))
            return Config(**data)
        except:
            raise IOError("Not able to find file at path: " + '"' + path + '"')

    def setup_env(self, env=None):
        if not env:
            os.environ["AWS_PROFILE"] = self.Enviroments[0].profile
            os.environ["AWS_DEFAULT_REGION"] = self.Enviroments[0].region


from .utils.logging import logger
from .utils.config import Config
import sys
import json
from .utils.common import create_change_set, remove_change_set, format_diff, get_yes_or_no, deploy_stack


def deploy(config_path, root_path):
    config = Config.parse(config_path)
    config.setup_env()
    services = config.Stacks
    logger.warning(f"Checking difference for stacks from file {config_path}")
    
    logger.info(f"* Found {len(services)} services checking differences...")

    for service in services:
        change_set = create_change_set(service.name, service.path, root_path, config)
        diffs = [format_diff(change)for change in change_set["Changes"]]

        if len(diffs):
            logger.warning(f"Found {len(diffs)} differences for the stack {service.name}")
            for diff in diffs:
                logger.warning(f"> {diff}")
            
            should_continue = get_yes_or_no(f"Do you wish to continue to update serivce: {service.name}")

            if not should_continue:
                remove_change_set(service.name, change_set["ChangeSetName"])
            else:
                logger.info("Deploying serivce...")
                deploy_stack(service.name, change_set["ChangeSetName"])
        else:
            logger.info(f"Found no differences for the stack {service.name}")

        



from .utils.logging import logger
from .utils.config import Config
import sys
from .utils.common import create_change_set, remove_change_set, format_diff


def diff(config_path, root_path):
    config = Config.parse(config_path)
    config.setup_env()
    
    services = config.Stacks

    logger.warning(f"Checking difference for stacks from file {config_path}")

    logger.info(f"* Found {len(services)} services checking differences...")

    for service in services:
        change_set = create_change_set(service.name, service.path, root_path, config)
        diffs = [format_diff(change)for change in change_set["Changes"]]

        if len(diffs):
            logger.warning(f"Found {len(diffs)} differences for the stack {service.name}")
            for diff in diffs:
                logger.warning(f"> {diff}")
        else:
            logger.info(f"Found no differences for the stack {service.name}")

        remove_change_set(service.name, change_set["ChangeSetName"])



import argparse
from .deploy import deploy
from .diff import diff
from .utils.logging import logger


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("action", choices=['diff', 'deploy', 'package', "version"], help='what action to preform')
    parser.add_argument("path", help='path to the file describing the services')
    parser.add_argument("-v", '--version', action='version', help='path to the file describing the services', version='aws-cf: 0.0.0')
    parser.add_argument("-r", "--root")

    args = parser.parse_args()

    try:
        if args.action == "deploy":
            deploy(args.path, args.root or ".")
        if args.action == "diff":
            diff(args.path, args.root or ".")
    except Exception as e:
        logger.error(str(e))





import sys
from utils.config import Config
from utils.common import package

if __name__ == "__main__":
    config_path = sys.argv[1]
    root_path = sys.argv[2]
    stack_name = sys.argv[3]

    config = Config.parse(config_path)
    stack = config.Stacks[0]
    yml = open(stack.path.replace("$root", root_path) + "/spec.yml").read()

    package(yml, config)
